<?php
class FW_Form
{
  protected $parameters = array();
  protected $elements = array();
  
  protected $errors = array();
  
  public function __construct($name, $method, $action)
  {
     if(strtolower($method) != "post" && strtolower($method) != "get")
     {
       throw new FW_Form_Exception("Der 2. Parameter des Konstruktors muss get oder post sein");
     }
     $this->parameters["name"] = $name;
     $this->parameters["method"] = $method;
     $this->parameters["action"] = $action;
  }
  
  public function setParameter($name, $value)
  {
    $name = strtolower($name);
    $this->parameters[$name] = $value;
  }
  
  public function getParameter($name)
  {
    $name = strtolower($name);
    
    if(isset($this->parameters[$name]))
    {
      return $this->parameters[$name];
    }
    return null;    
  }
  
  
  public function addElement(FW_Form_Element_Abstract $element, $position = null)
  {
    if($position === null || !is_int($position))
    {
      $this->elements[] = $element;
    }
    else
    {
      $this->elements[$position] = $element;
    }
    return $this;
  }
  
  public function getElementByPosition($position)
  {
    if(is_int($position) && isset($this->elements[$position]))
    {
      return $this->elements[$position];
    }
    else
    {
      throw new FW_Form_Exception("Diese Position hat einen ungültigen Wert");
    }
  }
  
  public function getElementByName($name)
  {
    foreach($this->elements as $element)
    {
      if($name == $element->getName())
      {
        return $element;
      }
    }
    return null;
  }
  
  public function validate()
  {
    $error_occured = false;
    foreach($this->elements as $element)
    {
      if(!$element->validate($this))
      {
        $this->errors[$element->getName()] = $element->getErrors();
        $error_occured = true;
      }
    }
    return !$error_occured;
  }
  
  public function getErrors()
  {
    return $this->errors;
  }
  
  public function addErrorMessages()
  {
    //Alle Elemente des Formulars durchlaufen
    foreach($this->elements as $element)
    {
      //Gibt es Fehler in diesem Element?
      if(isset($this->errors[$element->getName()]))
      {
          //Alle Fehler in diesem Element durchlaufen
          foreach($this->errors[$element->getName()] as $error)
          {
            //Fehler hinzufügen: FW_Form_Decorator_Error
            $element->addDecorator(new FW_Form_Decorator_Error($error));
          }
      }
    }
  }
  
  public function __toString()
  {
    //ksort ordnet das Array PHP-intern nach dem Index
    ksort($this->elements);
    
    $html = "<form";
    foreach($this->parameters as $name => $value)
    {
      $html .= " ".$name."='".$value."'";
    }
    $html .= ">";
    
    foreach($this->elements as $k => $element)
    {
      $html .= "\n ".$element->getHTML()."\n";
    }
    
    $html .="</form>";
     
    return $html;
  }
}
?>