<?php
abstract class FW_Form_Element_Abstract
{
  protected $validators = array();
  protected $decorators = array();
  protected $parameters = array();
  protected $error_msg = array();
  
  public function addValidator(FW_Validator_Abstract $validator, $error_msg)
  {
    $this->validators[] = array("validator" => $validator, "error_msg" => $error_msg);    
    return $this;
  }

  final public function getHTML()
  {
    $html = "";
    foreach($this->decorators as $decorator)
    {
      $html .= $decorator->getHTMlBefore($this);
    }
    $html .= $this->getHTMLofElement();
    foreach($this->decorators as $decorator)
    {
      $html .= $decorator->getHTMlAfter($this);
    }
    return $html; 
  }
  
  abstract public function getHTMLofElement();
  
  final public function validate($form_object)
  {
    $error_occured = false;
    foreach($this->validators as $validator)
    {
      $validator["validator"]->setValue("to_check", $this->getElementValue($form_object->getParameter("method")));
      if(!$validator["validator"]->isValid())
      {
        $this->error_msg[] = $validator["error_msg"];
        $error_occured = true;
      }
    }
    return !$error_occured;
  } 
  
  public function getErrors()
  {
    return $this->error_msg;
  }
  
  public function getElementValue($method)
  {
    $request_obj = FW_Http_Request::getInstance();
    
    switch($method)
    {
      case "post": return $request_obj->getPost($this->parameters["name"]);
      case "get": return $request_obj->getGet($this->parameters["name"]);
    }
    return $request_obj->getPost($this->parameters["name"]);
  }
  
  public function setParameter($name, $value)
  {
    $this->parameters[$name] = $value;
    return $this;
  }
  
  public function setName($name)
  {
    $this->setParameter("name", $name);
    return $this;
  }
  
  public function getName()
  {
    return $this->getParameter("name");
  }
  
  public function getID()
  {
    return $this->getParameter("id");
  }
  
  public function setID($id)
  {
    $this->setParameter("id", $id);
    return $this;
  }
  
  public function setValue($value)
  {
    $this->setParameter("value", $value);
    return $this;
  }
  
  public function getValue()
  {
    return $this->getParameter("value");
  }
  
  public function getParameter($name)
  {
     return (isset($this->parameters[$name])) ? $this->parameters[$name] : null;
  }
  
  protected function getFurtherParameters()
  {
    $html = "";
    foreach($this->parameters as $name => $value)
    {
      $html .= " ".$name."='".$value."'";
    }
    return $html;
  }
  
  public function addDecorator(FW_Form_Decorator_Abstract $decorator, $amount = 1)
  {
    for($i = 0; $i < (int)$amount; $i++)
    {
      $this->decorators[] = $decorator;
    }  
    return $this;
  }
}
?>