<?php
abstract class FW_Validator_Abstract
{
  protected $validators = array();
  protected $values = array();
  
  final public function isValid()
  {
    if(!isset($this->values["to_check"]))
    {      
      throw new FW_Validator_Exception("Der Wert 'to_check' muss angegeben werden und das zu überprüfende Element enthalten.");
    }
    
    
    //Erst prüfen, ob die hinzugefügten Validatoren alle true zurückgeben...
    foreach((array)$this->validators as $validator)
    {
      //... wenn nicht, abbruch der Schleife und Rückgabe von false
      if(!$validator->isValid())
      {
        return false;
      }
    }
  
  //Wenn die bisherigen Validatoren alle true ergaben, entscheidet der letzte (der "Elternvalidator"),
  //ob die Prüfung true oder false ergibt.
   return $this->isThisValid();
  }
  
  final public function addValidator(FW_Validator_Abstract $validator)
  {
    $validator->setValues($this->values);
    $this->validators[] = $validator;
  }
  
  abstract public function isThisValid();
  
  final public function setValues(array $values = array())
  {
    $this->values = array_merge($values, $this->values);
    //bei allen hinzugefügten validatoren werte mitteilen
    foreach((array)$this->validators as $validator)
    {
      $validator->setValues($this->values);
    }    
    return $this;
  }  
  
  final public function setValue($name, $value)
  {
    $this->values[$name] = $value;
    foreach((array)$this->validators as $validator)
    {
      $validator->setValue($name, $value);
    }
    return $this;
  }
  
  final public function __construct(array $values = array())
  {
    try
    {
      $this->setValues($values);
    }
    catch(FW_Validator_Exception $e)
    {
      throw $e;
    }
    $this->init();
  }  
  
  public function init()
  {
  }
}
?>